@@ -34,9 +34,9 @@ class gNMIclient(object):
     """
     This class instantiates the object, which interacts with the network elements over gNMI.
     """
-    def __init__(self, target: tuple, username: str = None, password: str = None, 
+    def __init__(self, target: tuple, username: str = None, password: str = None,
                  debug: bool = False, insecure: bool = False, path_cert: str = None, path_key: str = None, path_root: str = None, override: str = None,
-                 gnmi_timeout: int = 5, grpc_options: list = []):
+                 gnmi_timeout: int = 5, grpc_options: list = [], **kwargs ):
 
         """
         Initializing the object
@@ -60,6 +60,26 @@ class gNMIclient(object):
         else:
             self.__target = target
 
+        if 'interval_ms' in kwargs:
+            self.configureKeepalive( kwargs )
+
+    def configureKeepalive(self, interval_ms: int, timeout_ms: int = 20000,
+                           max_pings_without_data: int = 0,
+                           permit_without_calls: bool = True):
+        """
+        Helper method to set relevant client-side gRPC options to control keep-alive messages
+        Must be called before connect()
+
+        See https://github.com/grpc/grpc/blob/master/doc/keepalive.md
+
+        max_pings_without_data: default 0 to enable long idle connections
+        """
+        self.__options += [
+          ("grpc.keepalive_time_ms", interval_ms),
+          ("grpc.keepalive_timeout_ms", timeout_ms),
+          ("grpc.keepalive_permit_without_calls", 1 if permit_without_calls else 0),
+          ("grpc.http2.max_pings_without_data", max_pings_without_data),
+        ]
 
     def __enter__(self):
         """
@@ -75,8 +95,6 @@ class gNMIclient(object):
 
         if self.__insecure:
             self.__channel = grpc.insecure_channel(self.__target_path, self.__metadata + self.__options)
-            grpc.channel_ready_future(self.__channel).result(timeout=self.__gnmi_timeout)
-            self.__stub = gNMIStub(self.__channel)
 
         else:
             if self.__path_cert and self.__path_key and self.__path_root:
@@ -97,7 +115,7 @@ class gNMIclient(object):
                 except:
                     logger.error('The SSL certificate cannot be opened.')
                     raise Exception('The SSL certificate cannot be opened.')
-                    
+
             else:
                 try:
                     ssl_cert = ssl.get_server_certificate((self.__target[0], self.__target[1])).encode("utf-8")
@@ -110,15 +128,22 @@ class gNMIclient(object):
 
                 except:
                     logger.error(f'The SSL certificate cannot be retrieved from {self.__target}')
-                    raise Exception(f'The SSL certificate cannot be retrieved from {self.__target}') 
+                    raise Exception(f'The SSL certificate cannot be retrieved from {self.__target}')
 
-            self.__channel = grpc.secure_channel(self.__target_path, 
+            self.__channel = grpc.secure_channel(self.__target_path,
                                                  credentials=cert, options=self.__options)
-            grpc.channel_ready_future(self.__channel).result(timeout=self.__gnmi_timeout)
-            self.__stub = gNMIStub(self.__channel)
+
+        if self.__gnmi_timeout is None or self.__gnmi_timeout > 0:
+           self.wait_for_connect(self.__gnmi_timeout)
+        self.__stub = gNMIStub(self.__channel)
 
         return self
 
+    def wait_for_connect(self, timeout: int):
+        """
+        Wait for the gNMI connection to the server to come up, with given timeout
+        """
+        grpc.channel_ready_future(self.__channel).result(timeout=timeout)
 
     def capabilities(self):
         """
@@ -1064,6 +1089,27 @@ def telemetryParser(in_message=None, debug: bool = False):
 
                 response['update']['update'].append(update_container)
 
+            if in_message.update.delete:
+                response['update']['delete'] = []
+                for delete_msg in in_message.update.delete:
+                    delete_container = {}
+                    resource_path = []
+
+                    for path_elem in delete_msg.elem:
+                        tp = ''
+                        if path_elem.name:
+                            tp += path_elem.name
+
+                        if path_elem.key:
+                            # Use 'sorted' to have a consistent ordering of keys
+                            for pk_name, pk_value in sorted(path_elem.key.items()):
+                                tp += f'[{pk_name}={pk_value}]'
+
+                        resource_path.append(tp)
+
+                    delete_container.update({'path': '/'.join(resource_path)})
+                    response['update']['delete'].append(delete_container)
+
             return response
 
         elif in_message.HasField('sync_response'):
@@ -1075,4 +1121,3 @@ def telemetryParser(in_message=None, debug: bool = False):
         logger.error(f'Parsing of telemetry information is failed.')
 
         return None
-
